"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../util");
class RedisAdapter {
    constructor(redisClient) {
        this.clientReady = false;
        this.isPingingClient = false;
        this.redisClient = redisClient;
        this.clientReady = this.redisClient.ping();
        this.redisClient.on('ready', () => {
            this.clientReady = true;
        });
        this.redisClient.on('error', () => {
            this.clientReady = false;
        });
        this.checkIfReady();
    }
    ;
    /**
     * checkIfReady will return the last received ready status of the client.
     * If the client isn't ready, it will ping the redis client to check if it's ready
     * yet. This isn't a perfect solution, because we're not waiting for the response (so as to
     * not add latency to the underlying method calls). I believe it's a reasonable trade-off to
     * have a potential cache miss rather than add latency to all decorated method calls.
     */
    checkIfReady() {
        if (!this.clientReady && !this.isPingingClient) {
            this.isPingingClient = true;
            this.redisClient.ping(() => {
                this.clientReady = true;
                this.isPingingClient = false;
            });
        }
        return this.clientReady;
    }
    ;
    // Redis doesn't have a standard TTL, it's at a per-key basis
    getClientTTL() {
        return 0;
    }
    ;
    get(cacheKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const isReady = this.checkIfReady();
            if (isReady) {
                return new Promise((resolve, reject) => {
                    if (cacheKey.includes(':')) {
                        this.redisClient.hgetall(cacheKey, RedisAdapter.responseCallback(resolve, reject));
                    }
                    else {
                        this.redisClient.get(cacheKey, RedisAdapter.responseCallback(resolve, reject));
                    }
                }).then((result) => {
                    const usableResult = util_1.parseIfRequired(result);
                    if (usableResult && typeof usableResult === 'object' && Object.keys(usableResult).every(key => Number.isInteger(Number(key)))) {
                        return Object.keys(usableResult).map(key => util_1.parseIfRequired(usableResult[key]));
                    }
                    return usableResult;
                });
            }
            ;
            throw new Error('Redis client is not accepting connections.');
        });
    }
    ;
    /**
     * set - Sets a key equal to a value in a Redis cache
     *
     * @param cacheKey The key to store the value under
     * @param value    The value to store
     * @param ttl      Time to Live (how long, in seconds, the value should be cached)
     *
     * @returns {Promise}
     */
    set(cacheKey, value, ttl) {
        return __awaiter(this, void 0, void 0, function* () {
            const isReady = this.checkIfReady();
            if (isReady) {
                return new Promise((resolve, reject) => {
                    if (cacheKey.includes(':') && typeof value === 'object') {
                        const args = RedisAdapter.buildSetArgumentsFromObject(value);
                        this.redisClient.hmset(cacheKey, args, (err, result) => {
                            if (!err) {
                                // hmset doesn't add expiration by default, so we have to implement that here if ttl is given
                                if (ttl) {
                                    this.redisClient.expire(cacheKey, ttl, RedisAdapter.responseCallback(resolve, reject));
                                    return;
                                }
                            }
                            RedisAdapter.responseCallback(resolve, reject)(err, result);
                        });
                    }
                    else {
                        const usableValue = typeof value === 'string' ? value : JSON.stringify(value);
                        if (ttl) {
                            this.redisClient.set(cacheKey, usableValue, 'EX', ttl, RedisAdapter.responseCallback(resolve, reject));
                        }
                        else {
                            this.redisClient.set(cacheKey, usableValue, RedisAdapter.responseCallback(resolve, reject));
                        }
                    }
                });
            }
            throw new Error('Redis client is not accepting connections.');
        });
    }
    ;
    del(cacheKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const isReady = this.checkIfReady();
            if (isReady) {
                return new Promise((resolve, reject) => {
                    this.redisClient.del(cacheKey, RedisAdapter.responseCallback(resolve, reject));
                });
            }
            throw new Error('Redis client is not accepting connections.');
        });
    }
    ;
}
exports.RedisAdapter = RedisAdapter;
RedisAdapter.buildSetArgumentsFromObject = (objectValue) => Object.keys(objectValue).reduce((accum, curr) => {
    accum.push(curr, typeof objectValue[curr] === 'object' ? JSON.stringify(objectValue[curr]) : objectValue[curr]);
    return accum;
}, []);
RedisAdapter.responseCallback = (resolve, reject) => (err, response) => {
    if (err) {
        reject(err);
    }
    else {
        resolve(response);
    }
};
