"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const md5 = require("md5");
const serialize = require("serialize-javascript");
/**
 * extractKey - If data should be stored in a hash, this would be the name of the hash
 *
 * @param passedInKey The desired key, or function to build the key based on arguments/context
 * @param args        The arguments the decorated method was called with
 * @param context     The instance whose method is being called
 *
 * @returns {String}
 */
exports.extractKey = (passedInKey = '', args, context) => {
    // If the user passed in a cacheKey, use that. If it's a string, use it directly.
    // In the case of a function, we'll use the result of the called function.
    return passedInKey instanceof Function
        ? passedInKey(args, context)
        : passedInKey;
};
/**
 * getCacheKey - Determines the cache key to use. Either from an argument extractor function,
 *               string, or number that is passed in, or a hash based on args decorated function
 *               was called with (by default)
 *
 * @param methodName  The name of the method being called
 * @param passedInKey The desired key, or function to build the key based on arguments
 * @param args        The arguments the decorated method was called with
 * @param context     The instance whose method is being called
 *
 * @returns {String}
 */
exports.getCacheKey = (passedInKey = '', methodName, args, context) => {
    // If the user passed in a cacheKey, use that. If it's a string, use it directly.
    // In the case of a function, we'll use the result of the called function.
    if (passedInKey) {
        return exports.extractKey(passedInKey, args, context);
    }
    // Fall back to a default value (md5 hash of serialized arguments and context,
    // which is the instance the method was called from)
    const callMap = {
        args: args,
        methodName,
        context,
    };
    const serializedKey = serialize(callMap);
    const hashedKey = md5(serializedKey);
    return hashedKey;
};
exports.getFinalKey = (passedCacheKey = '', passedHashKey = '', methodName, args, context) => {
    const cacheKey = exports.getCacheKey(passedCacheKey, methodName, args, context);
    const hashKey = exports.extractKey(passedHashKey, args, context);
    return hashKey
        ? `${hashKey}:${cacheKey}`
        : cacheKey;
};
